@@ -1,18 +1,19 @@
 ### R code from vignette source 'updates.Rnw'
 
 ###################################################
-### code chunk number 1: updates.Rnw:20-26
+### code chunk number 1: updates.Rnw:20-27
 ###################################################
 library(spatstat)
 x <- read.dcf(file = system.file("DESCRIPTION", package = "spatstat"),
               fields = c("Version", "Date"))
 sversion <- as.character(x[,"Version"])
 sdate    <- as.character(x[,"Date"])
+sdevel   <- length(unlist(package_version(x[,"Version"]))) > 3 
 options(useFancyQuotes=FALSE)
 
 
 ###################################################
-### code chunk number 2: updates.Rnw:36-133
+### code chunk number 2: updates.Rnw:37-141
 ###################################################
 readSizeTable <- function(fname) {
   if(is.null(fname) || !file.exists(fname)) return(NULL)
@@ -21,8 +22,13 @@ readSizeTable <- function(fname) {
   return(a)
 }
 getSizeTable <- function(packagename="spatstat", tablename="packagesizes.txt") {
-  fname <- system.file("doc", tablename, package=packagename)
-  readSizeTable(fname)
+  fname <- system.file("info", tablename, package=packagename)
+  out <- readSizeTable(fname)
+  if(is.null(out)) {
+    fname <- system.file("doc", tablename, package=packagename)
+    out <- readSizeTable(fname)
+  }
+  return(out)
 }
 RemoveDevel <- function(sizetable) {
   ## remove entries with fractional version numbers
@@ -33,7 +39,7 @@ RemoveDevel <- function(sizetable) {
   return(st)
 }
 counts <- c("nhelpfiles", "nobjects", "ndatasets", "Rlines", "srclines")
-mergeSizeTables <- function(a, b, breakupdate, allow.devel=FALSE) {
+mergeSizeTables <- function(a, b, breakupdate, allow.devel=sdevel) {
   #' a is the running total for spatstat; b is a sub-package.
   #' breakupdate is the date when the code in b was removed from spatstat
   #' so that the size of 'b' must be added to 'a' for all dates >= breakupdate
@@ -72,6 +78,7 @@ z <- getSizeTable()
 ## installed sub-packages - access via the installed sub-packages
 zutils   <- getSizeTable("spatstat.utils")
 zdata    <- getSizeTable("spatstat.data")
+zunivar  <- getSizeTable("spatstat.univar")
 zsparse  <- getSizeTable("spatstat.sparse")
 zgeom    <- getSizeTable("spatstat.geom")
 zrandom  <- getSizeTable("spatstat.random")
@@ -97,6 +104,7 @@ z <- mergeSizeTables(z, zrandom, "2022-02-12")
 CoreSplitDay <- "2020-05-25"   # size of 'core' drops to 0 on this date
 z <- mergeSizeTables(z, zexplore, CoreSplitDay)
 z <- mergeSizeTables(z, zmodel,   CoreSplitDay)
+z <- mergeSizeTables(z, zunivar,  "2024-04-21")
 ## extension packages: these never overlapped spatstat
 z <- mergeSizeTables(z, zlocal)
 z <- mergeSizeTables(z, zgui)
@@ -106,7 +114,7 @@ currentcount <- z[nrow(z), counts]
 bookcount    <- z[z$version == "1.42-0", counts]
 changes <- currentcount - bookcount
 newobj <- changes[["nobjects"]]
-newdat <- changes[["ndatasets"]] + 1  # counting rule doesn't detect redwood3
+newdat <- changes[["ndatasets"]] + 2  # redwood3, shelling2
 newcode  <- changes[["Rlines"]] + changes[["srclines"]]
 bookcode <- bookcount[["Rlines"]] + bookcount[["srclines"]]
 currentcode <- currentcount[["Rlines"]] + currentcount[["srclines"]]
@@ -114,54 +122,67 @@ growth <- signif((100 * newcode)/bookcode, digits=2)
 
 
 ###################################################
-### code chunk number 3: updates.Rnw:145-150
+### code chunk number 3: updates.Rnw:153-159
 ###################################################
 options(SweaveHooks=list(fig=function() par(mar=0.2+c(2,4,2,0))))
-Plot <- function(fmla, ..., dat=z) {
+Plot <- function(fmla, ..., dat=z, ylim=NULL) {
   yvals <- eval(as.expression(fmla[[2]]), envir=dat)
-  plot(fmla, ..., data=dat, type="l", xlab="", lwd=2, ylim=c(0, max(yvals)))
+  if(is.null(ylim)) ylim <- c(0, max(yvals))
+  plot(fmla, ..., data=dat, type="n", xlab="", ylim=ylim, las=1)
 }
 
 
 ###################################################
-### code chunk number 4: updates.Rnw:156-161
+### code chunk number 4: updates.Rnw:165-179
 ###################################################
 getOption("SweaveHooks")[["fig"]]()
 Plot((Rlines + srclines)/1000 ~ date, ylab="Lines of code (x 1000)", 
      main="Spatstat growth")
-lines(srclines/1000 ~ date, data=z)
-text(as.Date("2015-01-01"), 9.5, "C code")
-text(as.Date("2015-01-01"), 60, "R code")
+with(z, {
+  polygon(c(date, rev(date)),
+          c(srclines, rep(0, length(srclines)))/1000,
+          angle=45, density=5, col="pink")
+  polygon(c(date, rev(date)),
+          c(srclines+Rlines, rev(srclines))/1000,
+          angle=135, density=5, col="lightblue")
+})
+text(as.Date("2019-01-01"), 90, "R code", col="blue")
+text(as.Date("2021-01-01"), 15, "C code", col="red")
+lines((Rlines + srclines)/1000 ~ date, data=z, lwd=3)
+lines(srclines/1000 ~ date, data=z, col="red")
 
 
 ###################################################
-### code chunk number 5: updates.Rnw:178-199
+### code chunk number 5: updates.Rnw:196-220
 ###################################################
 ## Tabulate latest version numbers of packages
 vtable <- data.frame(package="spatstat", version=sversion, date=as.Date(sdate))
-AppendVersion <- function(pkg, sizetable, v, allow.devel=FALSE) {
+AppendVersion <- function(pkg, sizetable, v, allow.devel=sdevel) {
   if(!allow.devel) sizetable <- RemoveDevel(sizetable)
   if(is.null(sizetable)) return(v)
   lastrow <- sizetable[nrow(sizetable), , drop=FALSE]
   if(is.null(lastrow)) return(v)
   rbind(v, data.frame(package=pkg, version=lastrow[,"version"], date=as.Date(lastrow[,"date"])))
 }
+vtable <- AppendVersion("spatstat.utils", zutils, vtable)
+vtable <- AppendVersion("spatstat.data", zdata, vtable)
+vtable <- AppendVersion("spatstat.sparse", zsparse, vtable)
+vtable <- AppendVersion("spatstat.univar", zunivar, vtable)
 vtable <- AppendVersion("spatstat.geom", zgeom, vtable)
 vtable <- AppendVersion("spatstat.random", zrandom, vtable)
 vtable <- AppendVersion("spatstat.explore", zexplore, vtable)
 vtable <- AppendVersion("spatstat.model", zmodel, vtable)
-##vtable <- AppendVersion("spatstat.core", zcore, vtable)
 vtable <- AppendVersion("spatstat.linnet", zlinnet, vtable)
-vtable <- AppendVersion("spatstat.sparse", zsparse, vtable)
-vtable <- AppendVersion("spatstat.data", zdata, vtable)
-vtable <- AppendVersion("spatstat.utils", zutils, vtable)
+## move spatstat to the bottom
+vtable <- rbind(vtable[-1, ], vtable[1, ])
+## add extras
 vtable <- AppendVersion("spatstat.local", zlocal, vtable)
 vtable <- AppendVersion("spatstat.Knet", zKnet, vtable)
 vtable <- AppendVersion("spatstat.gui", zgui, vtable)
 
 
 ###################################################
-### code chunk number 6: updates.Rnw:205-206
+### code chunk number 6: updates.Rnw:226-227
 ###################################################
 print(vtable[,c(3,1,2)], row.names=FALSE)
 
